@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.16'
+__version__ = '0.17'
 
 
 # These are reserved ipv6 addresses.
@@ -57,26 +57,43 @@ RECORD_RATIO = 20
 # Any access level used in the code should be listed here.
 # Access levels are represented as an unsigned small int in the database.
 ACCESS_LEVELS = {'dns_admin': 128,
-                 'domain_admin': 64,
+                 'unlocked_user': 64,
                  'user': 32,
                  'noop': 0}
 
-# Valid access rights for forward or reverse perms in the database.
-ACCESS_RIGHTS = ['rw', 'r']
-
 # This is a list of tables that are not audit logged when changes are made.
 # it is important not to overwrite these tables when doing a partial replay
 TABLES_NOT_AUDIT_LOGGED = ['audit_log', 'locks']
 
-# This is a list of ronly ecord types that can be modified by user level.
-USER_LEVEL_RECORDS = ['a', 'aaaa', 'ptr', 'cname']
-
 # This is a list of record types that can be indexed by IP address.
 RECORD_TYPES_INDEXED_BY_IP = ['ptr', 'a', 'aaaa']
 
 # This is the default config file location for roster server
 SERVER_CONFIG_FILE_LOCATION = '/etc/roster/roster_server.conf'
 
+# CONFIG_FILE_SCHMEA holds the expected inputs from config files and
+# their types for checking.
+# Supported data types: str, int, boolean, float
+CONFIG_FILE_SCHEMA = {'database': {'server': 'str', 'login': 'str',
+                                   'passwd': 'str', 'database': 'str',
+                                   'big_lock_timeout': 'int',
+                                   'big_lock_wait': 'int', 'ssl': 'boolean',
+                                   'ssl_ca': 'str', 'db_debug': 'boolean',
+                                   'db_debug_log': 'str'},
+                      'server': {'inf_renew_time': 'int', 'core_die_time': 'int',
+                                 'get_credentials_wait_increment': 'int',
+                                 'run_as_username': 'str',
+                                 'port': 'int', 'host': 'str',
+                                 'server_killswitch': 'boolean',
+                                 'lock_file': 'str', 'ssl_key_file': 'str',
+                                 'server_log_file': 'str', 'ssl_cert_file': 'str'},
+                      'credentials': {'authentication_method': 'str',
+                                      'exp_time': 'int'},
+                      'exporter': {'backup_dir': 'str',
+                                   'root_config_dir': 'str',
+                                   'named_dir': 'str'}}
+
+
 # The SUPPORTED_METHODS hash contains a hash for every supported method.
 # 'check' indicates whether the target zone/IP range should be checked.
 # 'write' indicates whether the method requires write access.
@@ -90,6 +107,11 @@ SUPPORTED_METHODS = {
                     {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['user']},
+    
+    'ListRecordsByZone':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['user']},
 
     'MakeRecord':   {'check': True,
                      'write': True,
@@ -118,7 +140,7 @@ SUPPORTED_METHODS = {
 
     'ListViews':    {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['user']},
 
     'GetViewsByUser':
                     {'check': False,
@@ -140,7 +162,7 @@ SUPPORTED_METHODS = {
     'ListViewAssignments':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeViewAssignment':
                     {'check': False,
@@ -155,7 +177,7 @@ SUPPORTED_METHODS = {
     'ListDnsServers':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeDnsServer':
                     {'check': False,
@@ -175,7 +197,7 @@ SUPPORTED_METHODS = {
     'ListDnsServerSets':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeDnsServerSet':
                     {'check': False,
@@ -195,7 +217,7 @@ SUPPORTED_METHODS = {
     'ListDnsServerSetViewAssignments':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeDnsServerSetViewAssignments':
                     {'check': False,
@@ -210,7 +232,7 @@ SUPPORTED_METHODS = {
     'ListDnsServerSetAssignments':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeDnsServerSetAssignments':
                     {'check': False,
@@ -234,7 +256,7 @@ SUPPORTED_METHODS = {
 
     'ListACLs':     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeACL':      {'check': False,
                      'write': True,
@@ -252,7 +274,7 @@ SUPPORTED_METHODS = {
     'ListViewToACLAssignments':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeViewToACLAssignments':
                     {'check': False,
@@ -267,7 +289,7 @@ SUPPORTED_METHODS = {
     'ListReverseRangeZoneAssignments':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['user']},
 
     'MakeReverseRangeZoneAssignment':
                     {'check': False,
@@ -282,7 +304,12 @@ SUPPORTED_METHODS = {
     'ListForwardZonePermissions':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['user']},
+
+    'UpdateGroupForwardPermission':
+                    {'check': False,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
 
     'MakeForwardZonePermission':
                     {'check': False,
@@ -297,7 +324,12 @@ SUPPORTED_METHODS = {
     'ListReverseRangePermissions':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['user']},
+
+    'UpdateGroupReversePermission':
+                    {'check': False,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
 
     'MakeReverseRangePermission':
                     {'check': False,
@@ -321,7 +353,7 @@ SUPPORTED_METHODS = {
 
     'ListZones':    {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['user']},
 
     'MakeZone':     {'check': False,
                      'write': True,
@@ -337,7 +369,7 @@ SUPPORTED_METHODS = {
 
     'ListUsers':    {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeUser':     {'check': False,
                      'write': True,
@@ -353,7 +385,7 @@ SUPPORTED_METHODS = {
 
     'ListGroups':   {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeGroup':    {'check': False,
                      'write': True,
@@ -370,7 +402,7 @@ SUPPORTED_METHODS = {
     'ListUserGroupAssignments':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeUserGroupAssignment':
                     {'check': False,
@@ -395,7 +427,7 @@ SUPPORTED_METHODS = {
     'ListReservedWords':
                     {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['domain_admin']},
+                     'access_level': ACCESS_LEVELS['unlocked_user']},
 
     'MakeReservedWord':
                     {'check': False,
@@ -490,7 +522,6 @@ TABLES = {
     'acls':              {'acl_name': 'UnicodeString'},
 
     'acl_ranges':        {'acl_ranges_acl_name': 'UnicodeString',
-                          'acl_range_allowed': 'IntBool',
                           'acl_range_cidr_block': 'CIDRBlock'},
 
     'views':             {'view_name': 'UnicodeString',
@@ -499,7 +530,8 @@ TABLES = {
 
     'view_acl_assignments':
                          {'view_acl_assignments_acl_name': 'UnicodeString',
-                          'view_acl_assignments_view_name': 'UnicodeString'},
+                          'view_acl_assignments_view_name': 'UnicodeString',
+                          'view_acl_assignments_range_allowed': 'IntBool'},
 
     'view_dependency_assignments':
         {'view_dependency_assignments_view_name': 'UnicodeString',
@@ -524,14 +556,22 @@ TABLES = {
          'user_group_assignments_user_name': 'UnicodeString'},
 
     'forward_zone_permissions':
-        {'forward_zone_permissions_group_name': 'UnicodeString',
-         'forward_zone_permissions_zone_name': 'UnicodeString',
-         'forward_zone_permissions_access_right': 'AccessRight'},
+        {'forward_zone_permissions_id': 'UnsignedInt',
+         'forward_zone_permissions_group_name': 'UnicodeString',
+         'forward_zone_permissions_zone_name': 'UnicodeString'},
+
+    'group_forward_permissions':
+        {'group_forward_permissions_forward_zone_permissions_id': 'UnsignedInt',
+         'group_forward_permissions_group_permission': 'GroupPermission'},
 
     'reverse_range_permissions':
-        {'reverse_range_permissions_group_name': 'UnicodeString',
-         'reverse_range_permissions_cidr_block': 'CIDRBlock',
-          'reverse_range_permissions_access_right': 'AccessRight'},
+        {'reverse_range_permissions_id': 'UnsignedInt',
+         'reverse_range_permissions_group_name': 'UnicodeString',
+         'reverse_range_permissions_cidr_block': 'CIDRBlock'},
+
+    'group_reverse_permissions':
+        {'group_reverse_permissions_reverse_range_permissions_id': 'UnsignedInt',
+         'group_reverse_permissions_group_permission': 'GroupPermission'},
 
     'reverse_range_zone_assignments':
         {'reverse_range_zone_assignments_zone_name': 'UnicodeString',
