@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.16'
+__version__ = '0.17'
 
 
 import constants
@@ -58,8 +58,8 @@ class CoreHelpers(object):
 
   ### These functions just expose helpers_lib functions for the 
   ### XML-RPC server. For doc strings see helpers_lib
-  def ListAccessRights(self):
-    return helpers_lib.ListAccessRights()
+  def ListGroupPermissions(self):
+    return self.db_instance.data_validation_instance.ListGroupPermissions()
 
   def ReverseIP(self, ip_address):
     return helpers_lib.ReverseIP(ip_address)
@@ -131,28 +131,29 @@ class CoreHelpers(object):
     return unicode(host_name)
 
   def AddFormattedRecords(self, zone_name, zone_file_string,
-                          view=u'any', zone_view=None,
-                          use_soa=True):
+                          view):
     """Adds records from a string of a partial zone file
 
     Inputs:
       zone_name: string of zone name
-      zone_file_string: string of zone name
+      zone_file_string: string of the file contents
       view: string of view name
-      zone_view: view to put SOA record in
-      use_soa: use a zone file with an SOA record
 
     Outputs:
       int: Amount of records added to db.
     """
-    if( use_soa ):
-      zone = dns.zone.from_text(str(zone_file_string))
-      origin = zone.origin
-    else:
-      origin = self.core_instance.ListZones(zone_name=zone_name)[
+    origin = self.core_instance.ListZones(zone_name=zone_name)[
           zone_name][view][u'zone_origin']
-      zone = dns.zone.from_text(
-          str(zone_file_string), check_origin=False, origin=origin)
+
+    zone = None
+    #If the file doesn't have an origin, we need to give it something
+    #otherwise dns.zone will raise an UnknownOrigin exception
+    if( zone_file_string.find("$ORIGIN") == -1 ):
+      zone = dns.zone.from_text(str(zone_file_string), check_origin=False,
+        origin=origin)
+    else:
+      zone = dns.zone.from_text(str(zone_file_string), check_origin=False)   
+
     make_record_args_list = []
     for record_tuple in zone.items():
       record_target = unicode(record_tuple[0])
@@ -225,24 +226,28 @@ class CoreHelpers(object):
                 'Unkown record type: %s.\n %s' % (
                     dns.rdatatype.to_text(record_object.rdtype), record_object))
 
-          view_dependency = u'any'
-          if( record_object.rdtype == dns.rdatatype.SOA ):
-            if( zone_view ):
-              view_dependency = zone_view
+          if( record_object.rdtype == dns.rdatatype.SOA and view == u'any'):
+            all_views = self.core_instance.ListZones(zone_name=zone_name)[zone_name]
+
+            for single_view in all_views:
+              if( single_view != u'any' ):
+                make_record_args_list.append(
+                  {u'record_type': record_type,
+                   u'record_target': record_target,
+                   u'record_zone_name': zone_name,
+                   u'record_arguments': record_args_dict,
+                   u'record_view_dependency': single_view,
+                   u'ttl': ttl})
           else:
-            if( view ):
-              view_dependency = view
-
-          make_record_args_list.append(
-              {u'record_type': record_type,
-               u'record_target': record_target,
-               u'record_zone_name': zone_name,
-               u'record_arguments': record_args_dict,
-               u'record_view_dependency': view_dependency,
-               u'ttl': ttl})
-
-    self.ProcessRecordsBatch(add_records=make_record_args_list)
+            make_record_args_list.append(
+                {u'record_type': record_type,
+                 u'record_target': record_target,
+                 u'record_zone_name': zone_name,
+                 u'record_arguments': record_args_dict,
+                 u'record_view_dependency': view,
+                 u'ttl': ttl})
 
+    self.ProcessRecordsBatch(add_records=make_record_args_list, zone_import=True)
     return len(make_record_args_list)
 
   def GetCIDRBlocksByView(self, view, username):
@@ -443,10 +448,10 @@ class CoreHelpers(object):
     Inputs:
       long_target: String of long PTR target
       view_name: String of view name
-    
+
     Raises:
       InvalidInputError: No suitable reverse range zone assignments found.
-    Ouptuts:
+    Outputs:
       string: String of short PTR target
     """
     if( not long_target.endswith('in-addr.arpa.') and not
@@ -506,15 +511,159 @@ class CoreHelpers(object):
     self.core_instance.RemoveRecord(record_type, target, zone_name,
                                     record_args_dict, view_name, ttl)
 
+  def MakeIPv4ClasslessReverseDelegation(self, name_server, cidr_block,
+                                     view_name=u'any', ttl=None):
+    """Creates classless forwarding for reverse DNS lookups
+
+    Inputs:
+      name_server: nameserver to add for reverse delegation
+      cidr_block: string of CIDR block
+      view_name: string of view for the reverse zone, defaults to 'any'
+      ttl: time-to-live for the newly added records, defaults to zone ttl
+
+    Raises:
+      InvalidInputError: nameserver required
+      InvalidInputError: cidr block range required
+      InvalidInputError: Not a valid zone name or CIDR block
+    """
+    view_dep = '%s_dep' % view_name
+    cidr_octets = cidr_block.split('.')
+    num_octets = len(cidr_octets)
+    if( num_octets != 4 ):
+      raise errors.InvalidInputError('Invalid CIDR octet number')
+    cidr_block_target = cidr_octets[-1]
+    broadcast_last_octet = cidr_block_target.split('/')[0]
+    netmask = cidr_block_target.split('/')[1]
+    if( str.isdigit(str(broadcast_last_octet)) and str.isdigit(str(netmask))):
+      if( int(netmask) < 25 or int(netmask) > 31 ):
+        raise errors.InvalidInputError('Invalid CIDR netmask: %s' % netmask)
+      if( int(broadcast_last_octet) < 0 or int(broadcast_last_octet) > 255):
+        raise errors.InvalidInputError('Invalid CIDR last octet')
+    else:
+      raise errors.InvalidInputError('Invalid CIDR last octet/netmask')
+
+    for i in range(1, len(cidr_octets) - 1):
+      if( str.isdigit(str(cidr_octets[i])) ):
+        if( int(cidr_octets[i]) < 0 or int(cidr_octets[i]) > 255):
+          raise errors.InvalidInputError('Invalid CIDR octet %s' %
+                                         cidr_octets[i])
+      else:
+          raise errors.InvalidInputError('Invalid CIDR octet %s' %
+                                         cidr_octets[i])
+
+    cidr_for_ipy = cidr_octets[0]
+    for i in range(1, num_octets - 1):
+      cidr_for_ipy = '%s.%s' % (cidr_for_ipy, cidr_octets[i])
+    cidr_for_ipy = '%s.%s/%s' % (cidr_for_ipy, '0', netmask)
+    expanded_cidr = self.CIDRExpand(cidr_for_ipy,
+                                    begin=long(broadcast_last_octet))
+    expanded_cidr.remove(expanded_cidr[-1])
+
+    zone_name = self.ListZoneByIPAddress(cidr_block)
+    if( zone_name is None ):
+      raise errors.InvalidInputError(
+          'ERROR: zone that includes cidr block %s not found' % cidr_block)
+
+    records = self.core_instance.ListRecords(zone_name=zone_name)
+    for record in records:
+      for ip in expanded_cidr:
+        if( self.ReverseIP(ip).split('.')[0] == record['target'] ):
+          raise errors.InvalidInputError('ERROR: existing record(s) with '
+                                         'target: %s overlaps given cidr: %s' %
+                                         (record['target'], cidr_block))
+
+    records_batch = []
+    cidr_last_target = int(broadcast_last_octet) + pow(2, 32 - int(netmask))
+    ns_target = cidr_block_target
+    ns_args_dict = self.core_instance.GetEmptyRecordArgsDict(u'ns')
+    ns_args_dict['name_server'] = name_server
+    ns_record = {'record_type': u'ns', 'record_target': ns_target,
+                 'record_zone_name': zone_name, 'view_name': view_name,
+                 'record_view_dependency': view_dep,
+                 'record_arguments': ns_args_dict}
+    records_batch.append(ns_record)
+
+    for ip in expanded_cidr:
+      reverse_ip = self.ReverseIP(ip)
+      split_reverse_ip = reverse_ip.split('.')
+      target = split_reverse_ip[0]
+      reverse_ip_for_record = '%s.%s' % (target, cidr_block_target)
+      for i in range(1, len(split_reverse_ip)):
+        reverse_ip_for_record = '%s.%s' % (reverse_ip_for_record,
+                                           split_reverse_ip[i])
+      if( target == broadcast_last_octet or target == unicode(
+             cidr_last_target - 1) ):
+        continue
+      cname_args_dict = self.core_instance.GetEmptyRecordArgsDict(u'cname')
+      cname_args_dict['assignment_host'] = unicode(reverse_ip_for_record)
+      cname_record = {'record_type': u'cname', 'record_target': unicode(target),
+                      'record_zone_name': zone_name, 'view_name': view_name,
+                      'record_view_dependency': view_dep,
+                      'record_arguments': cname_args_dict}
+      records_batch.append(cname_record)
+
+    self.ProcessRecordsBatch(add_records=records_batch)
+
+  def MakeIPv4ClasslessReverseDelegatedTargetZone(self, cidr_block):
+    """Creates a delegated reverse zone
+
+    Inputs:
+      cidr_block: string of IPv4 cidr block
+
+    Raises:
+      InvalidInputError: Not a valid cidr block
+    """
+    cidr_octets = cidr_block.split('.')
+    cidr_block_target = cidr_octets[-1]
+    broadcast_last_octet = cidr_block_target.split('/')[0]
+    netmask = cidr_block_target.split('/')[1]
+
+    if( str.isdigit(str(broadcast_last_octet)) and str.isdigit(str(netmask))):
+      if( int(netmask) < 25 or int(netmask) > 31 or
+          int(broadcast_last_octet) < 0 or int(broadcast_last_octet) > 255 ):
+        raise errors.InvalidInputError('Invalid CIDR block')
+    else:
+      raise errors.InvalidInputError('Invalid CIDR block')
+
+    for i in range(1, len(cidr_octets) - 1):
+      if( str.isdigit(str(cidr_octets[i])) ):
+        if( int(cidr_octets[i]) < 0 or int(cidr_octets[i]) > 255):
+          raise errors.InvalidInputError('Invalid CIDR block')
+      else:
+          raise errors.InvalidInputError('Invalid CIDR block')
+
+    zone_name = u'in-addr.arpa'
+    for i in range(0, len(cidr_octets)):
+      zone_name = u'%s.%s' % (cidr_octets[i], zone_name)
+    zone_type = u'master'
+    zone_origin = u'%s.' % zone_name
+    self.core_instance.MakeZone(zone_name, zone_type, zone_origin)
+
+  def ListAccessLevels(self):
+    """Lists access levels from constants for both integer and string keys
+
+    Outputs:
+      dict: dictionary of access levels with both string and integer-string keys
+
+    Example:
+      {'32': 32, '64': 64, '128': 128, 'user': 32, 'unlocked_user': 64,
+       'dns_admin': 128}
+    """
+    access_levels_dict = {}
+    for key, value in constants.ACCESS_LEVELS.iteritems():
+      access_levels_dict[str(value)] = value
+      access_levels_dict[key] = value
+    return access_levels_dict
+
   def ListAvailableIpsInCIDR(self, cidr_block, num_ips=1, view_name=None,
-                           zone_name=None):
+                             zone_name=None):
     """Finds first available ips. Only lists as many IPs as are available.
     Returns empty list if no IPs are available in given cidr block and a
     truncated list if only a portion of IPs are available.
 
     Inputs:
       cidr_block: string of ipv4 or ipv6 cidr block
-    
+
     Raises:
       InvalidInputError: IP is in a reserved IP space.
       InvalidInputError: Not a valid cidr block
@@ -545,13 +694,12 @@ class CoreHelpers(object):
       for ip in records[view]:
         taken_ips.append(ip)
     count = 0L
-    while( count < (cidr_block_ipy.len() - 1) ):
-      count += 1L
+    while( count < cidr_block_ipy.len() ):
       if( len(avail_ips) >= num_ips ):
         break
       if( cidr_block_ipy[count].strFullsize() not in taken_ips ):
         avail_ips.append(cidr_block_ipy[count].strFullsize())
-
+      count += 1L
     return avail_ips
 
   def ListRecordsByCIDRBlock(self, cidr_block, view_name=None, zone_name=None):
@@ -746,6 +894,145 @@ class CoreHelpers(object):
 
     return parsed_record_dict
 
+  def ListRecordsByZone(self, zone_name, view_name=None):
+    """Lists records in a given zone.
+
+    Inputs:
+      zone_name: name of the zone
+      view_name: name of the view
+    
+    Output:
+      dict: A dictionary Keyed by view, keyed by IP, listed by record.
+            example:
+                {u'test_view':
+                    {u'192.168.1.8':
+                        [{u'forward': True,
+                          u'host': u'host6.university.edu',
+                          u'zone': u'forward_zone',
+                          u'zone_origin': u'university.edu.'},
+                         {u'forward': False,
+                          u'host': u'host6.university.edu',
+                          u'zone': u'reverse_zone',
+                          u'zone_origin': u'1.168.192.in-addr.arpa.'}]}}
+    """
+    self.user_instance.Authorize('ListRecordsByZone')
+    record_list = {}
+    
+    records_dict = self.db_instance.GetEmptyRowDict('records')
+    zone_view_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'zone_view_assignments')
+    zone_dict = self.db_instance.GetEmptyRowDict('zones')
+    zone_dict['zone_name'] = zone_name
+    record_arguments_records_assignments_dict = (
+        self.db_instance.GetEmptyRowDict(
+            'record_arguments_records_assignments'))
+    ipv4_index_dict = self.db_instance.GetEmptyRowDict('ipv4_index')
+    ipv6_index_dict = self.db_instance.GetEmptyRowDict('ipv6_index')
+    if( view_name is not None and
+        view_name.endswith('_dep') or view_name == u'any' ):
+      records_dict['record_view_dependency'] = view_name
+    elif( view_name is not None ):
+      records_dict['record_view_dependency'] = '%s_dep' % view_name
+
+    args_ipv4 = ['zone_view_assignments', zone_view_assignments_dict,
+                 'zones', zone_dict, 'records', records_dict,
+                 'record_arguments_records_assignments',
+                 record_arguments_records_assignments_dict]
+    args_ipv6 = args_ipv4 + ['ipv6_index', ipv6_index_dict]
+    args_ipv4.append('ipv4_index')
+    args_ipv4.append(ipv4_index_dict)
+    self.db_instance.StartTransaction()
+    try:
+      record_list = self.db_instance.ListRow(*args_ipv4)
+      record_list = record_list + self.db_instance.ListRow(*args_ipv6)
+    finally:
+      self.db_instance.EndTransaction()
+    #Parsing Records
+    parsed_record_dict = {}
+    for record_entry in record_list:
+      if( record_entry[u'record_type'] not in 
+          constants.RECORD_TYPES_INDEXED_BY_IP ):
+        raise errors.IPIndexError('Record type not indexable by '
+                                  'IP: %s' % record_entry)
+      if( record_entry[u'record_view_dependency'].endswith('_dep') ):
+        record_view = record_entry[u'record_view_dependency'][:-4]
+      else:
+        record_view = record_entry[u'record_view_dependency']
+      if( record_view not in parsed_record_dict ):
+          parsed_record_dict[record_view] = {}
+      if( u'ipv4_dec_address' in record_entry ):
+        record_ip = u'%s' % (
+            IPy.IP(record_entry[u'ipv4_dec_address']).strNormal(1))
+        if( record_ip not in parsed_record_dict[record_view] ):
+          parsed_record_dict[record_view][record_ip] = []
+      elif( u'ipv6_dec_upper' in record_entry ):
+        decimal_ip = (
+            (record_entry[u'ipv6_dec_upper'] << 64) +
+            (record_entry[u'ipv6_dec_lower']) )
+        record_ip = u'%s' % IPy.IP(decimal_ip).strFullsize(0)
+        if( record_ip not in parsed_record_dict[record_view] ):
+          parsed_record_dict[record_view][record_ip] = []
+      else:
+        raise errors.IPIndexError(
+            'Record type unknown. Missing ipv4 or ipv6 dec index: %s' % (
+            record_entry))
+      record_item = {}
+      record_item['records_id'] = record_entry['records_id']
+      record_item['record_type'] = record_entry['record_type']
+      record_item['record_target'] = record_entry['record_target']
+      record_item['record_ttl'] = record_entry['record_ttl']
+      record_item['record_zone_name'] = record_entry['record_zone_name']
+      record_item[u'zone_origin'] = record_entry[u'zone_origin']
+      record_item['record_view_dependency'] = record_entry[
+                                              'record_view_dependency']
+      record_item['record_last_user'] = record_entry['record_last_user']
+      if record_entry[u'record_view_dependency'].endswith('_dep'):
+        record_item[u'view_name'] = record_entry[u'record_view_dependency'][:-4]
+      else:
+        record_item[u'view_name'] = record_entry[u'record_view_dependency']
+      if( record_entry[u'record_type'] == u'a' or
+          record_entry[u'record_type'] == u'aaaa' ):
+        record_item[u'forward'] = True
+        record_item[u'host'] = '%s.%s' % (
+            record_entry[u'record_target'],
+            record_entry[u'zone_origin'][:-1])
+        record_item[u'zone_origin'] = record_entry['zone_origin']
+        record_item[u'record_target'] = record_entry['record_target']
+        record_item[u'record_args_dict'] = {
+            'assignment_ip': record_entry['argument_value']}
+        parsed_record_dict[record_view][record_ip].append(record_item)
+      elif( record_entry[u'record_type'] == u'ptr' ):
+        record_item[u'zone_origin'] = record_entry['zone_origin']
+        record_item[u'record_target'] = record_entry['record_target']
+        record_item[u'forward'] = False
+        record_item[u'host'] = record_entry[u'argument_value'][:-1]
+        assignment_ip = helpers_lib.UnReverseIP(
+            '%s.%s' % (
+                record_entry['record_target'],record_entry['zone_origin']))
+        record_item[u'record_args_dict'] = {'assignment_ip': assignment_ip}
+        parsed_record_dict[record_view][record_ip].insert(0, record_item )
+    return parsed_record_dict
+
+  def SortRecordsByHost(self, records_dict):
+    """Generates an IP list sorted by record's host
+    Inputs:
+      record_dict: dictionary keyed by view, then keyed by IP
+                   dictionary from ListRecordsByCIDRBlock
+                   and from ListRecordsByZone
+
+    Outputs:
+      sorted_list: list of sorted records
+    """
+    sorted_list = []
+    target_sort = []
+    for view in records_dict:
+      for ip in records_dict[view]:
+        for record in records_dict[view][ip]:
+          target_sort.append(dict({'ip_address':IPy.IP(ip).strFullsize()}.items() + 
+                                  record.items()))
+    sorted_list = sorted(target_sort, key=lambda x: x['host'])
+    return sorted_list
+
   def ListNamedConfGlobalOptionsClient(self, option_id=None,
                                        dns_server_set=None, timestamp=None):
     """Converts XMLRPC datetime to datetime object and runs
@@ -759,8 +1046,6 @@ class CoreHelpers(object):
     Outputs:
       list: list of dictionarires from ListNamedConfGlobalOptions
     """
-    function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
-    self.user_instance.Authorize(function_name)
     success = False
     named_conf_global_options = None
 
@@ -871,7 +1156,36 @@ class CoreHelpers(object):
                                   current_args, success)
     return row_count
 
-  def ProcessRecordsBatch(self, delete_records = None, add_records = None):
+  def ConstructRecordArgsDictFromRecordID(self, record_id):
+    """Constructs the records_arg_dict from the Roster database given only 
+    the record id.
+
+    Inputs:
+    record_id: int of record id
+
+    Outputs:
+    record_args_dict: dictionary of arguments and their values
+    """
+
+    record_args_db_row = self.db_instance.GetEmptyRowDict(
+      'record_arguments_records_assignments')
+    record_args_db_row[
+        'record_arguments_records_assignments_record_id'] = record_id
+    record_args_db_dict = self.db_instance.ListRow(
+      'record_arguments_records_assignments', record_args_db_row)
+
+    record_args_dict = {}
+    for key in record_args_db_dict:
+      key_entry = key['argument_value']
+      if( key_entry.isdigit() ):
+        key_entry = int(key_entry)
+      record_args_dict[
+        key['record_arguments_records_assignments_argument_name']] = key_entry
+
+    return record_args_dict
+
+  def ProcessRecordsBatch(self, delete_records=None, add_records=None,
+                          zone_import=False):
     """Proccess batches of records
 
     Inputs:
@@ -905,18 +1219,15 @@ class CoreHelpers(object):
     changed_view_dep = []
     success = False
     try:
+      for record in add_records:
+        self.db_instance.ValidateRecordArgsDict(record[u'record_type'], 
+                                                record[u'record_arguments'])
+
       self.db_instance.StartTransaction()
       try:
         # REMOVE RECORDS
         for record in delete_records:
           record_dict = self.db_instance.GetEmptyRowDict('records')
-          self.user_instance.Authorize('ProcessRecordsBatch',
-              record_data = {
-                  'target': record['record_target'],
-                  'zone_name': record['record_zone_name'],
-                  'view_name': record_dict['record_view_dependency'],
-                  'record_type': record['record_type']},
-              current_transaction=True)
           record_dict['records_id'] = record['records_id']
           record_dict['record_type'] = record['record_type']
           record_dict['record_target'] = record['record_target']
@@ -930,6 +1241,18 @@ class CoreHelpers(object):
                 '%s_dep' % record['record_view_dependency'])
           record_dict['record_zone_name'] = record['record_zone_name']
           record_dict['record_last_user'] = record['record_last_user']
+          record_args_dict = self.ConstructRecordArgsDictFromRecordID(
+              record['records_id'])
+
+          self.user_instance.Authorize('ProcessRecordsBatch',
+              record_data = {
+                  'target': record['record_target'],
+                  'zone_name': record['record_zone_name'],
+                  'view_name': record_dict['record_view_dependency'],
+                  'record_type': record['record_type'],
+                  'record_args_dict': record_args_dict},
+              current_transaction=True)
+
           rows_deleted = self.db_instance.RemoveRow('records', record_dict)
           log_dict['delete'].append(record)
           row_count += 1
@@ -944,6 +1267,13 @@ class CoreHelpers(object):
 
         # ADD RECORDS
         for record in add_records:
+
+          #Target length check
+          if( not self.db_instance.data_validation_instance.isTarget(
+              record[u'record_target']) ):
+            raise errors.InvalidInputError('Target hostname is invalid. %s' % (
+                record[u'record_target']))
+
           view_name = record['record_view_dependency']
           if( not record['record_view_dependency'].endswith('_dep') and record[
                 'record_view_dependency'] != u'any'):
@@ -953,7 +1283,8 @@ class CoreHelpers(object):
                   'target': record['record_target'],
                   'zone_name': record['record_zone_name'],
                   'view_name': view_name,
-                  'record_type': record['record_type']},
+                  'record_type': record['record_type'],
+                  'record_args_dict': record['record_arguments']},
               current_transaction=True)
           if( record['record_type'] == u'ptr' ):
             if( record['record_arguments'][
@@ -1007,7 +1338,8 @@ class CoreHelpers(object):
               if( record['record_arguments'][arg] != current_record[arg] ):
                 break
             else:
-              raise errors.RecordsBatchError('Duplicate record found')
+              raise errors.RecordsBatchError('Duplicate record found: %s' %
+                                             current_record)
 
 
           records_dict['record_type'] = record['record_type']
@@ -1032,7 +1364,7 @@ class CoreHelpers(object):
                 record['record_arguments'])
         changed_view_dep = set(changed_view_dep)
         for view_dep_pair in changed_view_dep:
-          self.core_instance._IncrementSoa(*view_dep_pair)
+          self.core_instance._IncrementSoa(*view_dep_pair, missing_ok=zone_import)
 
       except:
         self.db_instance.EndTransaction(rollback=True)
@@ -1044,4 +1376,57 @@ class CoreHelpers(object):
                                   current_args, success)
     return row_count
 
+  def ListSortedHostsByZone(self, zone_name, view_name=None):
+    records_dict = self.ListRecordsByZone(zone_name, view_name=view_name)
+    sorted_records = self.SortRecordsByHost(records_dict)
+    hosts_dict = {}
+    for record in sorted_records:
+      direction = 'Reverse'
+      if( record['forward'] ):
+        direction = 'Forward'
+      if( not hosts_dict.has_key(record[u'view_name']) ):
+        hosts_dict.update({record[u'view_name']: []})
+      new_record = record.copy()
+      new_record['direction'] = direction
+      hosts_dict[record[u'view_name']].append(new_record)
+    return hosts_dict
+
+  def ListSortedHostsByCIDR(self, cidr, zone_name=None, view_name=None):
+    records_dict = self.ListRecordsByCIDRBlock(cidr, zone_name=zone_name, 
+      view_name=view_name)
+    ip_address_list = self.CIDRExpand(cidr)
+    
+    if ip_address_list is None:
+      ip_address_list = []
+    if( ip_address_list == [] ):
+      for view in records_dict:
+        ip_address_list.extend(records_dict[view].keys())
+      ip_address_list = list(set(ip_address_list))
+
+    hosts_dict = {}
+    if( len(records_dict) == 0 ):
+      hosts_dict['--'] = []
+      for ip_address in ip_address_list:
+        hosts_dict['--'].append(
+            {'host': '--', 'direction': '--',
+             'ip_address': ip_address, 'record_zone_name': '--'})
+    else:
+      for view in records_dict:
+        if( not hosts_dict.has_key(view) ):
+          hosts_dict.update({view: []})
+        for ip_address in ip_address_list:
+          if( ip_address in records_dict[view] ):
+            for record in records_dict[view][ip_address]:
+              direction = 'Reverse'
+              if( record['forward'] ):
+                direction = 'Forward'
+              new_record = record.copy()
+              new_record['direction'] = direction
+              new_record['ip_address'] = ip_address
+              hosts_dict[view].append(new_record)
+          else:
+            hosts_dict[view].append(
+                {'ip_address': ip_address, 'direction': '--', 'host': '--',
+                 'record_zone_name': '--'})
+    return hosts_dict
 # vi: set ai aw sw=2:
