@@ -33,9 +33,9 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.16'
-
+__version__ = '0.17'
 
+import sys
 import cPickle
 import datetime
 import re
@@ -46,21 +46,23 @@ import constants
 import errors
 import helpers_lib
 
+from roster_core import punycode_lib
 
 class DataValidation(object):
 
-  def __init__(self, reserved_words):
+  def __init__(self, reserved_words, group_permissions):
     self.reserved_words = reserved_words
+    self.group_permissions = group_permissions
 
   def isUnicodeString(self, u_string):
     """Checks that a string is unicode.
   
     Inputs:
       u_string: unicode string
-    
+ 
     Raises:
       ReservedWordError: Reserved word found, unable to complete request.
-    
+ 
     Outputs:
       bool: bool if string or not
     """
@@ -86,17 +88,19 @@ class DataValidation(object):
       return False
     return True
 
-  def isAccessRight(self, access_right):
-    """Checks to make sure that the string is a valid access right.
+  def isGroupPermission(self, group_permission):
+    """Checks to make sure that the string is a valid group permission.
   
     Inputs:
-      access_right: unicode string, and in constants.ACCESS_RIGHTS
+      group_permission: unicode string of a group permission
 
     Outputs:
-      bool: if access right is valid or not
+      bool: if group permission is valid or not
     """
-    if( self.isUnicodeString(access_right) and access_right in
-        constants.ACCESS_RIGHTS ):
+    group_permissions_list = [
+        permission.lower() for permission in self.group_permissions]
+    if( self.isUnicodeString(group_permission) and
+        group_permission.lower() in group_permissions_list ):
       return True
     return False
 
@@ -207,6 +211,25 @@ class DataValidation(object):
       return True
     return False
 
+  def isTarget(self, target):
+    """Checks that a target and it's components have the correct length
+
+    Inputs:
+    target: target string
+
+    Outputs:
+    bool: if it is a valid target"""
+
+    target = punycode_lib.Uni2Puny(unicode(target))
+    
+    if( len(target) > 255 ):
+      return False
+
+    for component in target.split('.'):
+      if( len(component) > 63 ):
+        return False
+
+    return True
 
   def isHostname(self, host_name):
     """Checks that is a unicode string and that is properly dotted.
@@ -222,7 +245,8 @@ class DataValidation(object):
     if( self.isUnicodeStringNoSpaces(host_name) and
         host_name.endswith('.') and
         host_name.split('.') > 2 and
-        not host_name.startswith('.') ):
+        not host_name.startswith('.') and
+        self.isTarget(host_name) ):
       return True
     return False
 
@@ -240,6 +264,19 @@ class DataValidation(object):
       return True
     return False
 
+  def isUnicodeString255(self, string):
+    """Checks that is a unicode string and that is less than 256 characters
+    long
+
+    Inputs:
+      string: string to validate
+
+    Outputs:
+      bool: if it is a valid unicode string of correct length"""
+
+    if( self.isUnicodeString(string) and len(string) < 256 ):
+      return True
+    return False
 
   def isDateTime(self, date_time):
     """Checks that is a unicode string and that is a valid time stamp.
@@ -270,6 +307,13 @@ class DataValidation(object):
       return False
     return True
 
+  def ListGroupPermissions(self):
+    """Returns a list of group permissions pulled from the database
+
+    Outputs:
+      list: list of string group permissions
+    """
+    return self.group_permissions
 
   def ValidateRowDict(self, table_name, row_dict, none_ok=False,
                       all_none_ok=False):
@@ -290,7 +334,6 @@ class DataValidation(object):
       UnexpectedDataError: Need to fill out at least one value in dict
     """
     main_dict = helpers_lib.GetRowDict(table_name)
-
     for key in main_dict.iterkeys():
       if( key not in row_dict ):
         raise errors.UnexpectedDataError('Missing key %s in dictionary' % key)
@@ -303,9 +346,8 @@ class DataValidation(object):
       if( not 'is%s' % main_dict[key] in dir(self) ):
         raise errors.FunctionError('No function to check data '
                                           'type: %s' % main_dict[key])
-
       if( not getattr(self, 'is%s' % main_dict[key])(value) ):
-        if( (not none_ok and not key.endswith('_id')) or 
+        if( (not none_ok and not key.endswith('_id')) or
             (none_ok and value is not None) ):
           raise errors.UnexpectedDataError('Invalid data type %s for %s: %s' % (
               main_dict[key], key, value))
